<?php
namespace Teknisa\SM\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00004109NegociacaoContratoService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Row;

class Sm00004109NegociacaoContratoController {
    
    protected $negociacaoservice;
    protected $environment;
    
    public function __construct(Sm00004109NegociacaoContratoService $negociacaoservice, Environment $environment) {
        $this->negociacaoservice = $negociacaoservice;
        $this->environment      = $environment;
    }
    
    /**
     * Insere os dados de Negociação de Contrato
    **/
    public function saveDadosNegContrato(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $existeParcelaApurada = $this->negociacaoservice->saveDadosNegContrato($row, $nrorg, $cdoperador);
            
            $response->addDataSet(new DataSet("dados", array("existeParcelaApurada" => $existeParcelaApurada)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Insere os dados de Negociação de Contrato
    **/
    public function editParcelasNegoContr(Request\Row $request, Response $response) {
        try {
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $row             = $request->getRow();
            $parcelas        = $row["PARCELAS"];
            $dadosNegociacao = $row["NEGOCIACAO"];
            
            $existeParcelaApurada = $this->negociacaoservice->editParcelasNegoContr($parcelas, $dadosNegociacao, $nrorg, $cdoperador);
            $response->addDataSet(new DataSet("retorno", array("existeParcelaApurada" => $existeParcelaApurada)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Adiciona parcelas de Negociação de Contrato
    **/
    public function adicionarParcelasNegContr(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $existeParcelaApurada = $this->negociacaoservice->adicionarParcelasNegContr($row, $nrorg, $cdoperador);
            
            $response->addDataSet(new DataSet("dados", array("existeParcelaApurada" => $existeParcelaApurada)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * Insere os dados de Negociação de Contrato
    **/
    public function recebeMinMaxParcelas(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrnegcontrato = $row["NRNEGCONTRATO"];
            $dtini         = $row["DTINICIO"];
            $dtfin         = $row["DTFINAL"];
            $nrorg         = $this->environment->getNrOrg();
            
            //Valida se o período que está sendo inserido já existe em outra negociação com as mesmas variáveis para o mesmo cliente
            $this->negociacaoservice->validarParcelaPeriodoExistenteEmOutraNegociacao($nrnegcontrato, $dtini, $dtfin, $nrorg);
            
            $result = $this->negociacaoservice->recebeMinMaxParcelas($nrnegcontrato, $nrorg);
            $response->addDataSet(new DataSet("dados", array("parcelas" => $result)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     *Exclui a parcela de negociação de contrato
    **/
    public function excluirParcela(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrparcelasnegcontr = $row["NRPARCELASNEGCONTR"];
            $nrnegcontrato      = $row["NRNEGCONTRATO"];
            $nrorg              = $this->environment->getNrOrg();
            
            $this->negociacaoservice->excluirParcela($nrparcelasnegcontr, $nrnegcontrato, $nrorg);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     *Exclui a parcela de negociação de contrato
    **/
    public function gerarParcelasNegContratoWiz(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $datainicial       = $row["DTINICIO"];
            $datafinal         = $row["DTFINAL"];
            $idmultiplicaqtmed = $row["IDMULTIPLICAQTMED"];
            $valor             = $row["VRPARCELANEGOCIA"];
            
            $result = $this->negociacaoservice->gerarParcelasNegContratoWiz($datainicial, $datafinal, $idmultiplicaqtmed, $valor);
            $response->addDataSet(new DataSet("dados", array("parcelas" => $result)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se ja existe um registro de negociação com os mesmos dados da que esta sendo inserida.
    **/
    public function validarNegociacoesInseridas(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrorg = $this->environment->getNrOrg();
            
            $result = $this->negociacaoservice->validarNegociacoesInseridas($row, $nrorg);
            $response->addDataSet(new DataSet("dados", array("valida" => $result)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     *Exclui a negociação de contrato de Apuração de Faturamento
    **/
    public function excluirNegociacaoApura(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrorg   = $this->environment->getNrOrg();
            
            $result = $this->negociacaoservice->excluirNegociacaoApura($row, $nrorg);
            $response->addDataSet(new DataSet("excluirNegociacaoApura", array("resultado" => $result)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}


?>