<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00029000LocalAtendimentoFactory as LocalAtendFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\ParceiroNegocioService;
use Teknisa\SM\Service\EnderecoParceiroService;
use Teknisa\Model\Geral\Entities\Enderecoparc;
use Teknisa\SM\Factory\Sm00005000ParametrizacoesServicesFactory;
use Teknisa\SM\Service\IntegracoesTeknisaService;

class Sm00029000LocalAtendimentoService {
    
    protected $entityManager;
    protected $connection;
    protected $LocalAtendFactory;
    protected $environment;
    protected $parceiroNegocioService;
    protected $enderecoParceiroService;
    protected $parametrizacaoServicesFactory;
    protected $integracoesTeknisaService;

    public function __construct(EntityManager $entityManager, LocalAtendFactory $LocalAtendFactory, environment $environment, NewCode $newCode, ParceiroNegocioService $parceiroNegocioService, EnderecoParceiroService $enderecoParceiroService, Sm00005000ParametrizacoesServicesFactory $parametrizacaoServicesFactory, IntegracoesTeknisaService $integracoesTeknisaService) {
        $this->entityManager                 = $entityManager;
        $this->connection                    = $entityManager->getConnection();
        $this->LocalAtendFactory             = $LocalAtendFactory;
        $this->environment                   = $environment;
        $this->newCode                       = $newCode;
        $this->parceiroNegocioService        = $parceiroNegocioService;
        $this->enderecoParceiroService       = $enderecoParceiroService;
        $this->parametrizacaoServicesFactory = $parametrizacaoServicesFactory;
        $this->integracoesTeknisaService     = $integracoesTeknisaService;
    }
    
   /**
     * A função "verificaTipoHierarqOrg" Verifica se existe o tipo de Hierarquia Padrão.
     */
     
     public function verificaTipoHierarqOrg($nrorg){
    
        $params = array(
            "NRORG"  => $nrorg
        );
        
        $tpHierOrg = $this->LocalAtendFactory->verificaTipoHierarqOrg($params);
        
        return $tpHierOrg;
    }
    
    
     /**
     * Insere as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function inserirTudo($row) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            $row["NRINSCRICAOPARC"] = preg_replace('/[^0-9]/', '', $row["NRINSCRICAOPARC"]);

            if (isset($row["NRCEPENDERECO"])) {
                $row["NRCEPENDERECO"] = str_replace(array(".", "-"), "", $row["NRCEPENDERECO"]);
            }
            
            /* Tipo de Hierarquia */
            $tpHierOrg = $this->verificaTipoHierarqOrg($nrorg);
            
            if(count($tpHierOrg) > 0){ /* Se ja tiver passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $tpHierOrg[0]["NRTIPOHIERARQORG"];
            }
            else{ /* Se não tiver cria e passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $this->inserirTipoHierarquiaOrg($row);
            }
           
            /* Parceiro de Negócio */ 
            $parcnegocio = $this->inserirParcNegocio($row);
            
            //Parceria de Estrutura
            $this->inserirParceria($parcnegocio, "ESTRUTURA");
            
            /* Endereço do Parceiro de Negócio */
            $nrenderecoparc = $this->inserirEnderecoParc($row, $parcnegocio);
            
            if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {
                $existeInscricao = $this->verificarExisteClienteComMesmaInscricao($parcnegocio, $row["NRINSCRICAOPARC"], $nrorg);
                if ($existeInscricao) {
                    throw new \Exception("Operação bloqueada! <br>Já existe um cliente ativo com esse CNPJ.");
                }
                else {
                    $paramsReplicaParceiro = array(
                        "TIPODEPARCEIRO"  => array("ESTRUTURA", "CLIENTE"),
                        "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
                        "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
                        "NMPRINCIPALPARC" => $row["NMESTRUTURAM"],
                        "NMSECUNDARIPARC" => $row["NMESTRUTURAM"],
                        "NRPARCNEGOCIO"   => $parcnegocio,
                        "IDATIVO"         => "S"
                    );
                    
                    $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
                    
                    if (!empty($nrenderecoparc)) {
                        $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);
                        
                        $enderecoparc = $enderecoparcRepository->findOneBy(array(
                            "nrenderecoparc" => $nrenderecoparc,
                            "nrorg" => $nrorg
                        ));
                        
                        $paramsReplicaParceiro = array(
                            "TIPODEPARCEIRO" => array("ESTRUTURA", "CLIENTE")
                        );
                        $this->enderecoParceiroService->replicaEnderecoParceiro($enderecoparc, $paramsReplicaParceiro);
                    }
                }
            }
            
            /* Estrutura M */
            $estruturam = $this->inserirEstruturaM($row, $parcnegocio);
            
            /* Estrutura H */
            $this->inserirEstruturaH($row, $parcnegocio, $estruturam); 
            
            /* Hierarquia */
            $this->inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg);

            /* Chamar APi do ERP para criar filial */
            $chamouIntegracaoERP = null;
            if (isset($row["IDCONVERTIDOFILIAL"]) && $row["IDCONVERTIDOFILIAL"] == "S") {
                $this->verificarFilialRepetidaOutroLocal($row["CDFILIAL"], $estruturam);

                $chamouIntegracaoERP = $this->converterLocalEmFilial($row);
                if ($chamouIntegracaoERP) {
                    $this->atualizarFilialLocalAtendimento($row["CDFILIAL"], $estruturam);
                }
            }
            
            $this->connection->commit();
            
            return [
                "NRESTRUTURAM"        => $estruturam,
                "chamouIntegracaoERP" => $chamouIntegracaoERP,
                "parcnegocio"         => $parcnegocio,
            ];
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Edita as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function editarTudo($row) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            $row["NRINSCRICAOPARC"] = preg_replace('/[^0-9]/', '', $row["NRINSCRICAOPARC"]);

            if (isset($row["NRCEPENDERECO"])) {
                $row["NRCEPENDERECO"] = str_replace(array(".", "-"), "", $row["NRCEPENDERECO"]);
            }
            
            /* Parceiro de Negócio */ 
            $this->editarParcNegocio($row);
            
            /* Endereço do Parceiro de Negócio */
            $params = array(
                "NRENDERECOPARC" => $row["NRENDERECOPARC"]
            );
            $dataset = $this->LocalAtendFactory->getEnderecoParceiro($params);
            if (count($dataset) > 0) {
                $this->editarEnderecoParc($row);
            }
            else {
                $this->inserirEnderecoParc($row, $row["NRPARCNEGOCIO"]);
            }
            
            $paramsReplicaParceiro = array(
                "TIPODEPARCEIRO"  => array("ESTRUTURA", "CLIENTE"),
                "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
                "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
                "NMPRINCIPALPARC" => $row["NMESTRUTURAM"],
                "NMSECUNDARIPARC" => $row["NMESTRUTURAM"],
                "NRPARCNEGOCIO"   => $row["NRPARCNEGOCIO"],
                "IDATIVO"         => "S"
            );
                    
            if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {
                $existeInscricao = $this->verificarExisteClienteComMesmaInscricao($row["NRPARCNEGOCIO"], $row["NRINSCRICAOPARC"], $nrorg);
                if ($existeInscricao) {
                    throw new \Exception("Operação bloqueada! <br>Já existe um cliente ativo com esse CNPJ.");
                }
                else {
                    $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
                    
                    $nrenderecoparc = $row["NRENDERECOPARC"];
                
                    if (!empty($nrenderecoparc)) {
                        $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);
                        
                        $enderecoparc = $enderecoparcRepository->findOneBy(array(
                            "nrenderecoparc" => $nrenderecoparc,
                            "nrorg" => $nrorg
                        ));
                        
                        $paramsReplicaParceiro = array(
                            "TIPODEPARCEIRO" => array("ESTRUTURA", "CLIENTE")
                        );
                        $this->enderecoParceiroService->replicaEnderecoParceiro($enderecoparc, $paramsReplicaParceiro);
                    }
                }
            }
            else {
                $paramsReplicaParceiro["TIPODEPARCEIRO"] = array("ESTRUTURA");
                $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
            }

            /* Estrutura M */
            $this->editarEstruturaM($row);
            
            /* Estrutura H */
            $this->editarEstruturaH($row); 
            
            /* Hierarquia */
            $this->editarHierarquiaOrg($row);

            /* Chamar APi do ERP para criar filial */
            $chamouIntegracaoERP = null;
            if (isset($row["IDCONVERTIDOFILIAL"]) && $row["IDCONVERTIDOFILIAL"] == "S") {
                $this->verificarFilialRepetidaOutroLocal($row["CDFILIAL"], $row["NRESTRUTURAM"]);

                $chamouIntegracaoERP = $this->converterLocalEmFilial($row);
                if ($chamouIntegracaoERP) {
                    $this->atualizarFilialLocalAtendimento($row["CDFILIAL"], $row["NRESTRUTURAM"]);
                }
            }
            else {
                //Desvincular filial com o local de atendimento
                $this->atualizarFilialLocalAtendimento(null, $row["NRESTRUTURAM"]);
            }
            
            $this->connection->commit();

            return $chamouIntegracaoERP;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Exclui as tabelas HierarquiaOrg, EstruturaH, EstruturaM, EndercoParc e ParcNegocio
     */
    public function deletarTudo($row) {
        $nrestruturam = $row['NRESTRUTURAM'];
        $nrorg = $this->environment->getNrOrg();
         
        $params = array(
            "NRESTRUTURAM" => $nrestruturam,
            "NRORG"        => $nrorg
        );
        
        $filhos = $this->LocalAtendFactory->recebefilhos($params);
        
        $this->connection->beginTransaction();

        try {
            
            foreach($filhos as $filho){
                
                $estruturam = $filho['NRESTRUTURAM'];
                $parcnegocio = $filho['NRPARCNEGOCIO'];
                
                /* Hierarquia */
                $this->deletarHierarquiaOrg($estruturam);
                
                /* Estrutura H */
                $this->deletarEstruturaH($estruturam); 
                
                /* Estrutura M */
                $this->deletarEstruturaM($estruturam);
                
                /* Endereço do Parceiro de Negócio */
                $this->deletarEnderecoParc($parcnegocio);
                
                /* Parceiro de Negócio */ 
                $this->deletarParcNegocio($parcnegocio);
            }

            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Salva o Tipo de Hierarquia.
     */
    public function inserirTipoHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("TIPOHIERARQORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRTIPOHIERARQORG"  => $novoCodigo,
            "NMTIPOHIERARQORG"  => 'Padrão - Local de Atendimento SM',
            "NRTIPOESTRUTSUP"   => $row["NRTIPOESTRUTURA"],
            "NRTIPOESTRUTREL"   => $row["NRTIPOESTRUTURA"],
            "NRORG"             => $nrorg,
            "DTINCLUSAO"        => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperador,
            "IDATIVO"           => "S"
        );
        
        $this->LocalAtendFactory->inserirTipoHierarquiaOrg($params);
        
        return $novoCodigo;
    }
    
     /**
     * Salva o Parceiro de Negócio.
     */
    public function inserirParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create(); 
        
        $cdtipoparcprincipal = "ESTRUTURA";
        if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S")
            $cdtipoparcprincipal = "CLIENTE";
        
        $novoCodigo = $this->newCode->getCodeByTableName("PARCNEGOCIO", $nrorg, 1, 10, true);
        
        $params = array(
            "NRPARCNEGOCIO"       => $novoCodigo,
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     => $row["NRINSCRICAOPARC"],
            "CDTIPOPARCPRINCIPAL" => $cdtipoparcprincipal,
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "IDPESSOAFISICA"      => "N",
            "IDINSTITUICAO"       => "S",
            "IDPARCFUNDIDO"       => "N",
            "NRORG"               => $nrorg,
            "DTINCLUSAO"          => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"       => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador,
            "IDATIVO"             => "S"
        );
        
        $this->LocalAtendFactory->inserirParcNegocio($params);
        
        return $novoCodigo;
    }
    
    public function inserirParceria($parcnegocio, $cdtipoparceiro) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $nrparceria = $this->newCode->getCodeByTableName("PARCERIA", $nrorg, 1, 10, true);
        $params = array(
            "NRPARCERIA"     => $nrparceria,
            "NRPARCNEGOCIO"  => $parcnegocio,
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "NRORG"          => $nrorg,
            "IDATIVO"        => "S",
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador
        );
        
        $this->LocalAtendFactory->inserirParceria($params);
    }
    
    /**
     * atualiza o Parceiro de Negócio.
     */
    public function editarParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $cdtipoparcprincipal = "ESTRUTURA";
        if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {
            $cdtipoparcprincipal = "CLIENTE";
        }
        
        $params = array(
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     => $row["NRINSCRICAOPARC"],
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "CDTIPOPARCPRINCIPAL" => $cdtipoparcprincipal,
            "CDOPERULTATU"        => $cdoperador,
            "NRORG"               => $nrorg,
            "NRPARCNEGOCIO"       => $row["NRPARCNEGOCIO"]
        );
        
        $this->LocalAtendFactory->editarParcNegocio($params);
    }
    
     /**
     * Deleta o Parceiro de Negócio.
     */
    public function deletarParcNegocio($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarParcNegocio($params);
    }

     /**
     * Salva a Estrutura M.
     */
    public function inserirEstruturaM($row, $parcnegocio) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
            
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAM", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAM"     => $novoCodigo,
            "NMESTRUTURAM"     => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA"  => $row["NRTIPOESTRUTURA"],
            "DTINIVIGENCIA"    => $dataAtual->format('d/m/Y'),         
            "NRPARCNEGOCIO"    => $parcnegocio,
            "CDCENTCUST"       => $row["CDCENTCUST"],
            //"DSEMAILESTRUTURA" => $row["DSEMAILESTRUTURA"],
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaM($params);
        
        return $novoCodigo;
    }    
    
     /**
     * Atualiza a Estrutura M.
     */
    public function editarEstruturaM($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"     => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA"  => $row["NRTIPOESTRUTURA"],
            "CDCENTCUST"       => $row["CDCENTCUST"],
            "DSEMAILESTRUTURA" => $row["DSEMAILESTRUTURA"],
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRESTRUTURAM"     => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaM($params);
    }   
    
    /**
     * Deleta a Estrutura M.
     */
    public function deletarEstruturaM($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaM($params);
    }   
    
    /**
     * Salva a Estrutura H.
     */
    public function inserirEstruturaH($row, $parcnegocio, $estruturam) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAH", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAH"   => $novoCodigo,
            "NMESTRUTURAH"   => $row["NMESTRUTURAM"],
            "NRESTRUTURAM"   => $estruturam,
            "DTMESCOMPETENC" => $dataAtual->format('d/m/Y'),
            "NRPARCNEGOCIO"  => $parcnegocio,
            "NRORG"          => $nrorg,
            "DTINCLUSAO"     => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "IDATIVO"        => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaH($params);
    }
    
    /**
     * Atualiza a Estrutura H.
     */
    public function editarEstruturaH($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"   => $row["NMESTRUTURAM"],
            "CDOPERULTATU"   => $cdoperador,
            "NRORG"          => $nrorg,
            "NRESTRUTURAH"   => $row["NRESTRUTURAH"],
            "NRESTRUTURAM"   => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaH($params);
    }
    
    /**
     * Deleta a Estrutura H.
     */
    public function deletarEstruturaH($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaH($params);
    }
    
     /**
     * Salva a Hierarquia da Estrutura.
     */
    public function inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("HIERARQUIAORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRHIERARQUIAORG"  => $novoCodigo,
            "NRTIPOHIERARQORG" => $tipoHierarqOrg,
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $estruturam,
            "DTINIVIGENCIA"    => $dataAtual->format('d/m/Y'),
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirHierarquiaOrg($params);
    }
    
     /**
     * atualiza a Hierarquia da Estrutura.
     */
    public function editarHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $row["NRESTRUTURAM"], 
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRHIERARQUIAORG"  => $row["NRHIERARQUIAORG"]
        );
        
        $this->LocalAtendFactory->editarHierarquiaOrg($params);
    }
    
     /**
     * Deleta a Hierarquia da Estrutura.
     */
    public function deletarHierarquiaOrg($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarHierarquiaOrg($params);
    }
    
    /**
     * Salva o endereço de Hierarquia da Estrutura.
     */
    public function inserirEnderecoParc($row, $parcnegocio) {
        // $nrorg = $this->environment->getNrOrg();
        // $cdoperador = $this->environment->getCdOperador();
        // $dataAtual = date_create();
        
        // $novoCodigo = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
        
        // $params = array(
        //     "NRENDERECOPARC"   => $novoCodigo,
        //     "NRPARCNEGOCIO"    => $parcnegocio, 
        //     //"CDTIPOENDERECO"   => "PRINCIPAL",
        //     //"DSENDERECO"       => $row["DSENDERECO"],
        //    // "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
        //     //"DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
        //     //"CDPAIS"           => $row["CDPAIS"],
        //    // "SGESTADO"         => $row["SGESTADO"],
        //    // "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
        //    // "CDBAIRRO"         => $row["CDBAIRRO"],
        //     //"NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
        //     //"NRCEPENDERECO"    => $row["NRCEPENDERECO"],
        //    // "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
        //     "NRORG"            => $nrorg,
        //     "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
        //     "NRORGINCLUSAO"    => $nrorg,
        //     "CDOPERINCLUSAO"   => $cdoperador,
        //     "IDATIVO"          => "S"
        // );
        
        // $this->LocalAtendFactory->inserirEnderecoParc($params);
        
        // return $novoCodigo;
    }
    
    /**
     * atualiza o endereço de Hierarquia da Estrutura.
     */
    public function editarEnderecoParc($row) {
        // $nrorg = $this->environment->getNrOrg();
        // $cdoperador = $this->environment->getCdOperador();

        // $params = array(
        //     "DSENDERECO"       => $row["DSENDERECO"],
        //     "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
        //     "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
        //     "CDPAIS"           => $row["CDPAIS"],
        //     "SGESTADO"         => $row["SGESTADO"],
        //     "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
        //     "CDBAIRRO"         => $row["CDBAIRRO"],
        //     "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
        //     "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
        //     "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
        //     "CDOPERULTATU"     => $cdoperador,
        //     "NRORG"            => $nrorg,
        //     "NRENDERECOPARC"   => $row["NRENDERECOPARC"],
        //     "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
        // );
        
        // $this->LocalAtendFactory->editarEnderecoParc($params);
    }
    
     /**
     * Deleta o endereço de Hierarquia da Estrutura.
     */
    public function deletarEnderecoParc($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEnderecoParc($params);
    }
    
    public function verificarExisteClienteComMesmaInscricao($nrparcnegocio, $nrinscricaoparc, $nrorg) {
        $params = array(
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "NRORG"           => $nrorg,
            "NRPARCNEGOCIO"   => $nrparcnegocio
        );
        
        $dataset = $this->LocalAtendFactory->verificarExisteClienteComMesmaInscricao($params);
        return ((int)$dataset[0]["QTD"] > 0);
    }

    public function converterLocalEmFilial($row) {
        $chamouIntegracao = false;
        $urlIntegracaoErp = $this->getUrlIntegracaoErpFilial();

        if (!empty($urlIntegracaoErp)) {
            $isNew = true;
            $filial = $this->getFilial($row["CDFILIAL"]);
            if (!is_null($filial)) {
                $isNew = false;
            }

            $tiposInscricao = [
                "CNPJ"   => "J",
                "CPF"    => "F",
                "CODIGO" => "O",
                "LIVRE"  => "I"
            ];

            $dadosOperador = $this->getDadosOperador();
            $email = $dadosOperador["DSEMAILOPER"];
            $senha = $dadosOperador["CDSENHAOPERWEB"];
    
            $dadosIntegracao = [
                "__is_new"      => $isNew,
                "changed"       => true,
                "CDFILIAL"      => $row["CDFILIAL"],
                "NMFILIAL"      => substr($row["NMESTRUTURAM"], 0, 30),
                "NMRAZSOCFILI"  => $row["NMESTRUTURAM"],
                "IDTPIJURFILI"  => $tiposInscricao[$row["CDTIPOINSCRICAO"]],
                "NRINSJURFILI"  => $row["NRINSCRICAOPARC"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "NMRAZSOCEMPR"  => $row["NMEMPRESA"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "CDINSCMUNI"    => $row["CDINSCMUNI"],
                "IDTPENDEFILI"  => "P",
                "CDPAIS"        => $row["CDPAIS"],
                "SGESTADO"      => $row["SGESTADO"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO"],
                "DSENDEFILISEP" => $row["DSENDERECO"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO"],
                "NRCEPFILI"     => $row["NRCEPENDERECO"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO"],
                "IDFUNCAOFILI"  => null
            ];

            $chamouIntegracao = true;
            try {
                $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilial($urlIntegracaoErp, $dadosIntegracao, $email, $senha);
            }
            catch (\Exception $e) {
                throw new \Exception("Falha na integração com o ERP:<br>" . $e->getMessage());
            }
        }
        
        return $chamouIntegracao;
    }

    public function getUrlIntegracaoErpFilial() {
        $nrorg = $this->environment->getNrOrg();
        $params = ["NRORG" => $nrorg];
        $dataset = $this->parametrizacaoServicesFactory->getUrlIntegracaoErpFilial($params);
        return $dataset[0]["URLINTEGRACAOERPFILIAL"];
    }

    public function getDadosOperador() {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();

        $params = [
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        ];
        $dataset = $this->LocalAtendFactory->getDadosOperador($params);
        return $dataset[0];
    }

    public function getFilial($cdfilial) {
        $params = [
            "CDFILIAL" => $cdfilial
        ];
        
        $dataset = $this->LocalAtendFactory->getFilial($params);
        return count($dataset) > 0 ? $dataset[0] : null;
    }

    public function atualizarFilialLocalAtendimento($cdfilial, $nrestruturam) {
        $params = [
            "CDFILIAL"     => $cdfilial,
            "NRESTRUTURAM" => $nrestruturam
        ];

        $this->LocalAtendFactory->atualizarFilialLocalAtendimento($params);
    }

    /**
     * Valida se a filial já está relacionada a um outro local de atendimento
     */
    public function verificarFilialRepetidaOutroLocal($cdfilial, $nrestruturam) {
        $params = [
            "CDFILIAL"     => $cdfilial,
            "NRESTRUTURAM" => $nrestruturam
        ];

        $dataset = $this->LocalAtendFactory->getQtdLocalAtendimentoPorFilial($params);
        if ((int)$dataset[0]["QTD"] > 0) {
            throw new \Exception("FILIAL_VINCULADA_OUTRO_LOCAL");
        }
    }

    public function findEnderecos($parametros) {
        $param = array(
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"]
        );
        
        $dataset = $this->LocalAtendFactory->findEnderecos($param);
        return $dataset;
    }

    public function preparaDadosEnderecos($parametros) {
        $principal = array();
        $cobranca = array();
        $entrega = array();
        $comercial = array();
       
            foreach ($parametros['row']['ENDERECOS'] as $chave => $valor) {
                if (strpos($chave, '_principal') !== false) {
                    $chave = str_replace('_principal', '', $chave);
                    $principal[$chave] = $valor;
                    $principal["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "PRINCIPAL");
                    $principal["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $principal["CDTIPOENDERECO"] = "PRINCIPAL";
                    ($retorno["NOVO"]) ? $principal["insert"] = true : $principal["insert"] = false;

                } elseif (strpos($chave, '_cobranca') !== false) {
                    $chave = str_replace('_cobranca', '', $chave);
                    $cobranca[$chave] = $valor;
                    $cobranca["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "COBRANCA");
                    $cobranca["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $cobranca["CDTIPOENDERECO"] = "COBRANCA";
                    ($retorno["NOVO"]) ? $cobranca["insert"] = true : $cobranca["insert"] = false;

                } elseif (strpos($chave, '_entrega') !== false){
                    $chave = str_replace('_entrega', '', $chave);
                    $entrega[$chave] = $valor;
                    $entrega["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "ENTREGA");
                    $entrega["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $entrega["CDTIPOENDERECO"] = "ENTREGA";
                    ($retorno["NOVO"]) ? $entrega["insert"] = true : $entrega["insert"] = false;
                    
                } else if(strpos($chave, '_comercial') !== false) {
                    $chave = str_replace('_comercial', '', $chave);
                    $comercial[$chave] = $valor;
                    $comercial["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "COMERCIAL");
                    $comercial["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $comercial["CDTIPOENDERECO"] = "COMERCIAL";
                    ($retorno["NOVO"]) ? $comercial["insert"] = true : $comercial["insert"] = false;
                }
            }

            $enderecos = array($principal, $cobranca, $entrega, $comercial);
            foreach ($enderecos as $values) {
                $this->inserirAtualizarEnredecos($values);
            }
    }

    public function inserirAtualizarEnredecos($param) {
       
        $this->connection->beginTransaction();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        try {
            
            $params = array(
                "NRENDERECOPARC"   => isset($param["NRENDERECOPARC"]) ? $param["NRENDERECOPARC"] : null,
                "NRPARCNEGOCIO"    => isset( $param['NRPARCNEGOCIO']) ?  $param['NRPARCNEGOCIO'] : null,
                "CDTIPOENDERECO"   => isset($param["CDTIPOENDERECO"]) ? $param["CDTIPOENDERECO"] : null,
                "NRORG"            => $nrorg,
                "IDATIVO"          => "S",
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "CDPAIS"           => isset($param["CDPAIS"]) ? $param["CDPAIS"] : null,
                "SGESTADO"         => isset($param["SGESTADO"]) ? $param["SGESTADO"] : null,
                "CDMUNICIPIO"      => isset($param["CDMUNICIPIO"]) ? $param["CDMUNICIPIO"] : null,
                "CDBAIRRO"         => isset($param["CDBAIRRO"]) ?  $param["CDBAIRRO"] : null,
                "NMBAIRROENDERECO" => isset($param["NMBAIRROENDERECO"]) ? $param["NMBAIRROENDERECO"] : null,
                "DSCOMPLEENDERECO" => isset($param["DSCOMPLEENDERECO"]) ? $param["DSCOMPLEENDERECO"] : null ,
                "NRCEPENDERECO"    => isset($param["NRCEPENDERECO"]) ? str_replace(array(".", "-"), "", $param["NRCEPENDERECO"]) : null,
                "DSENDERECO"       => isset($param["DSENDERECO"]) ? $param["DSENDERECO"] : null,
                "NRIMOVELENDERECO" => isset($param["NRIMOVELENDERECO"]) ? $param["NRIMOVELENDERECO"] : null,
                "DSREFERENCIAENDE" => isset($param["DSREFERENCIAENDE"]) ? $param["DSREFERENCIAENDE"] : null,
                );
                
                if(isset($param["insert"]) && $param["insert"]) {
                     $this->LocalAtendFactory->inserirEnderecos($params);
                } else {
                    $this->LocalAtendFactory->atualizarEnderecos($params);
                }
                
            $this->connection->commit();
        }
        catch (\Exception $e) {
            throw new \Exception("Falha ao atualizar o endereço" . $e->getMessage());
        }
        
    }

    public function verificarSeExisteCadastroEndereco($nrparcneg, $tipo){
        $nrorg = $this->environment->getNrOrg();
        $retorno = $this->LocalAtendFactory->verificarSeExisteCadastroEndereco($nrparcneg, $tipo);
        
        if(isset($retorno[0])){
            $retornoData = array(
                "NRENDERECOPARC" => $retorno[0]["NRENDERECOPARC"],
                "NOVO" => false,
            );
        } else {
            $retornoData = array(
                "NRENDERECOPARC" => $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true),
                "NOVO" => true,
            );
        }
       return $retornoData;
    }

}

?>
