function CrosstabService(maskEngine, ZhUtil) {

    var applyMask = function(mask, value) {
        return mask ? Util.applyMask(value, mask, maskEngine, ZhUtil) : value;
    };

	var _buildDataSet = function(widget, dataSet, crosstabDataSet, index) {
		crosstabDataSet = crosstabDataSet || [];
		index = index || 0;

		var yAxisValue = widget.crosstabRows[index];
		var crossRow = {};

		widget.crosstabColumns.forEach(function(column){
			crossRow[column.key] = {
				label: widget.emptyLabel || ""
			};
		});
		crossRow[widget.yAxisProperty] = {
			__isYAxisProperty: true,
			label: yAxisValue
		};

		var filteredDataSet = dataSet.filter(function(row){
			if(row[widget.yAxisProperty] === yAxisValue){
				var property = row[widget.xAxisProperty] ? widget.xAxisProperty : widget.xAxisLabelProperty;
				crossRow[row[property]] = {
					__row: row,
					label: applyMask(widget.labelPropertyMask, row[widget.labelProperty])
				};
			}
			return row[widget.yAxisProperty] !== yAxisValue;
		});
		crosstabDataSet.push(crossRow);

		if(index < (widget.crosstabRows.length - 1) && filteredDataSet.length > 0){
			_buildDataSet(widget, filteredDataSet, crosstabDataSet, ++index);
		}
		return crosstabDataSet;
	};

	var _clearColumnsAndRows = function(widget) {
		widget.crosstabColumns = [];
		widget.crosstabRows = [];
	};

	var _buildCrosstabColumn = function(row, xAxisProperty, xAxisLabelProperty) {
		return {
			key: row[xAxisProperty] || row[xAxisLabelProperty],
			label: row[xAxisLabelProperty] || row[xAxisProperty]
		};
	};

	var _buildColumnsAndRows = function(widget, dataSet) {
		var crosstabColumns = {};
		var crosstabRows = {};

		widget.crosstabColumns.push({ key: widget.yAxisProperty, label: widget.yAxisLabel });

		dataSet.forEach(function(row){
			if(!crosstabColumns[row[widget.xAxisProperty]]){
				crosstabColumns[row[widget.xAxisProperty]] = true;
				widget.crosstabColumns.push(_buildCrosstabColumn(row, widget.xAxisProperty, widget.xAxisLabelProperty));
			}

			if(!crosstabRows[row[widget.yAxisProperty]]){
				crosstabRows[row[widget.yAxisProperty]] = true;
				widget.crosstabRows.push(row[widget.yAxisProperty]);
			}
		});
	};

	var _widgetIsValid = function(widget){
		return widget && widget.xAxisProperty && widget.yAxisProperty && widget.labelProperty;
	};

	this.build = function(widget, dataSet) {
		if(!_widgetIsValid(widget)){
			throw "Crosstab service: Invalid arguments!";
		}
		_clearColumnsAndRows(widget);
		var crosstabDataSet = [];
		if(Array.isArray(dataSet) && dataSet.length) {
			_buildColumnsAndRows(widget, dataSet);
			crosstabDataSet = _buildDataSet(widget, dataSet);
		}
		return crosstabDataSet;
	};

}